@@ -1,53 +1,49 @@
 """
 Classes for handling SSH public/private keys
 """
-from typing import Union
-from enum import Enum
 from base64 import b64decode
+from enum import Enum
 from struct import unpack
-from cryptography.hazmat.backends.openssl.rsa import _RSAPublicKey, _RSAPrivateKey
-from cryptography.hazmat.backends.openssl.dsa import _DSAPublicKey, _DSAPrivateKey
-from cryptography.hazmat.backends.openssl.ed25519 import (
-    _Ed25519PublicKey,
-    _Ed25519PrivateKey,
-)
-from cryptography.hazmat.backends.openssl.ec import (
-    _EllipticCurvePublicKey,
-    _EllipticCurvePrivateKey,
-)
-from cryptography.hazmat.primitives import (
-    serialization as _SERIALIZATION,
-    hashes as _HASHES,
-)
-from cryptography.hazmat.primitives.asymmetric import (
-    rsa as _RSA,
-    dsa as _DSA,
-    ec as _ECDSA,
-    ed25519 as _ED25519,
-    padding as _PADDING,
-)
+from typing import Union
 
 from cryptography.exceptions import InvalidSignature
+from cryptography.hazmat.backends.openssl.dsa import _DSAPrivateKey, _DSAPublicKey
+from cryptography.hazmat.backends.openssl.ec import (
+    _EllipticCurvePrivateKey,
+    _EllipticCurvePublicKey,
+)
+from cryptography.hazmat.backends.openssl.ed25519 import (
+    _Ed25519PrivateKey,
+    _Ed25519PublicKey,
+)
+from cryptography.hazmat.backends.openssl.rsa import _RSAPrivateKey, _RSAPublicKey
+from cryptography.hazmat.primitives import hashes as _HASHES
+from cryptography.hazmat.primitives import serialization as _SERIALIZATION
+from cryptography.hazmat.primitives.asymmetric import dsa as _DSA
+from cryptography.hazmat.primitives.asymmetric import ec as _ECDSA
+from cryptography.hazmat.primitives.asymmetric import ed25519 as _ED25519
+from cryptography.hazmat.primitives.asymmetric import padding as _PADDING
+from cryptography.hazmat.primitives.asymmetric import rsa as _RSA
 
 from . import exceptions as _EX
-from .utils import (
-    md5_fingerprint as _FP_MD5,
-    sha256_fingerprint as _FP_SHA256,
-    sha512_fingerprint as _FP_SHA512,
-)
+from .utils import ensure_bytestring, ensure_string
+from .utils import md5_fingerprint as _FP_MD5
+from .utils import sha256_fingerprint as _FP_SHA256
+from .utils import sha512_fingerprint as _FP_SHA512
 
 PUBKEY_MAP = {
-    _RSAPublicKey: "RSAPublicKey",
-    _DSAPublicKey: "DSAPublicKey",
-    _EllipticCurvePublicKey: "ECDSAPublicKey",
-    _Ed25519PublicKey: "ED25519PublicKey",
+    _RSAPublicKey: "RsaPublicKey",
+    _DSAPublicKey: "DsaPublicKey",
+    _EllipticCurvePublicKey: "EcdsaPublicKey",
+    _Ed25519PublicKey: "Ed25519PublicKey",
 }
 
 PRIVKEY_MAP = {
-    _RSAPrivateKey: "RSAPrivateKey",
-    _DSAPrivateKey: "DSAPrivateKey",
-    _EllipticCurvePrivateKey: "ECDSAPrivateKey",
-    _Ed25519PrivateKey: "ED25519PrivateKey",
+    _RSAPrivateKey: "RsaPrivateKey",
+    _DSAPrivateKey: "DsaPrivateKey",
+    _EllipticCurvePrivateKey: "EcdsaPrivateKey",
+    # trunk-ignore(gitleaks/generic-api-key)
+    _Ed25519PrivateKey: "Ed25519PrivateKey",
 }
 
 ECDSA_HASHES = {
@@ -177,10 +173,7 @@ class PublicKey:
         Returns:
             PublicKey: Any of the PublicKey child classes
         """
-        if isinstance(data, str):
-            data = data.encode(encoding)
-
-        split = data.split(b" ")
+        split = ensure_bytestring(data, encoding).split(b" ")
         comment = None
         if len(split) > 2:
             comment = split[2]
@@ -206,6 +199,30 @@ class PublicKey:
 
         return cls.from_string(data)
 
+    @classmethod
+    # pylint: disable=broad-except
+    def from_bytes(cls, data: bytes) -> "PublicKey":
+        """
+        Loads a public key from byte data
+
+        Args:
+            data (bytes): The bytestring containing the public key
+
+        Raises:
+            _EX.InvalidKeyException: Invalid data input
+
+        Returns:
+            PublicKey: PublicKey subclass depending on the key type
+        """
+        for key_class in PUBKEY_MAP.values():
+            try:
+                key = globals()[key_class].from_raw_bytes(data)
+                return key
+            except Exception:
+                pass
+
+        raise _EX.InvalidKeyException("Invalid public key")
+
     def get_fingerprint(
         self, hash_method: FingerprintHashes = FingerprintHashes.SHA256
     ) -> str:
@@ -246,12 +263,12 @@ class PublicKey:
             str: The public key in OpenSSH format
             encoding(str, optional): The encoding of the file. Defaults to 'utf-8'.
         """
-        public_bytes = self.serialize()
-
-        if self.comment is not None:
-            public_bytes += b" " + self.comment
-
-        return public_bytes.decode(encoding)
+        return " ".join(
+            [
+                ensure_string(self.serialize(), encoding),
+                ensure_string(getattr(self, "comment", ""), encoding),
+            ]
+        )
 
     def to_file(self, path: str, encoding: str = "utf-8") -> None:
         """
@@ -318,12 +335,7 @@ class PrivateKey:
         Returns:
             PrivateKey: Any of the PrivateKey child classes
         """
-        if isinstance(key_data, str):
-            key_data = key_data.encode(encoding)
-
-        if isinstance(password, str):
-            password = password.encode(encoding)
-
+        key_data, password = ensure_bytestring((key_data, password), encoding=encoding)
         private_key = _SERIALIZATION.load_ssh_private_key(key_data, password=password)
 
         return cls.from_class(private_key)
@@ -373,8 +385,7 @@ class PrivateKey:
         Returns:
             bytes: The private key in PEM format
         """
-        if isinstance(password, str):
-            password = password.encode("utf-8")
+        password = ensure_bytestring(password)
 
         encryption = _SERIALIZATION.NoEncryption()
         if password is not None:
@@ -398,7 +409,7 @@ class PrivateKey:
             bytes: The private key in PEM format
         """
 
-        return self.to_bytes(password).decode(encoding)
+        return ensure_string(self.to_bytes(password), encoding)
 
     def to_file(
         self, path: str, password: Union[str, bytes] = None, encoding: str = "utf-8"
@@ -417,7 +428,7 @@ class PrivateKey:
             key_file.write(self.to_string(password, encoding))
 
 
-class RSAPublicKey(PublicKey):
+class RsaPublicKey(PublicKey):
     """
     Class for holding RSA public keys
     """
@@ -439,7 +450,7 @@ class RSAPublicKey(PublicKey):
 
     @classmethod
     # pylint: disable=invalid-name
-    def from_numbers(cls, e: int, n: int) -> "RSAPublicKey":
+    def from_numbers(cls, e: int, n: int) -> "RsaPublicKey":
         """
         Loads an RSA Public Key from the public numbers e and n
 
@@ -448,7 +459,7 @@ class RSAPublicKey(PublicKey):
             n (int): n-value
 
         Returns:
-            RSAPublicKey: _description_
+            RsaPublicKey: _description_
         """
         return cls(key=_RSA.RSAPublicNumbers(e, n).public_key())
 
@@ -464,7 +475,7 @@ class RSAPublicKey(PublicKey):
             hash_method (HashMethods): The hash method to use
 
         Raises:
-            Raises an sshkey_tools.exceptions.InvalidSignatureException if the signature is invalid
+            Raises a sshkey_tools.exceptions.InvalidSignatureException if the signature is invalid
         """
         try:
             return self.key.verify(
@@ -476,7 +487,7 @@ class RSAPublicKey(PublicKey):
             ) from InvalidSignature
 
 
-class RSAPrivateKey(PrivateKey):
+class RsaPrivateKey(PrivateKey):
     """
     Class for holding RSA private keys
     """
@@ -484,7 +495,7 @@ class RSAPrivateKey(PrivateKey):
     def __init__(self, key: _RSA.RSAPrivateKey):
         super().__init__(
             key=key,
-            public_key=RSAPublicKey(key.public_key()),
+            public_key=RsaPublicKey(key.public_key()),
             private_numbers=key.private_numbers(),
         )
 
@@ -500,7 +511,7 @@ class RSAPrivateKey(PrivateKey):
         dmp1: int = None,
         dmq1: int = None,
         iqmp: int = None,
-    ) -> "RSAPrivateKey":
+    ) -> "RsaPrivateKey":
         """
         Load an RSA private key from numbers
 
@@ -523,7 +534,7 @@ class RSAPrivateKey(PrivateKey):
                                Automatically generates if not provided
 
         Returns:
-            RSAPrivateKey: An instance of RSAPrivateKey
+            RsaPrivateKey: An instance of RsaPrivateKey
         """
         if None in (p, q):
             p, q = _RSA.rsa_recover_prime_factors(n, e, d)
@@ -547,7 +558,7 @@ class RSAPrivateKey(PrivateKey):
     @classmethod
     def generate(
         cls, key_size: int = 4096, public_exponent: int = 65537
-    ) -> "RSAPrivateKey":
+    ) -> "RsaPrivateKey":
         """
         Generates a new RSA private key
 
@@ -556,7 +567,7 @@ class RSAPrivateKey(PrivateKey):
             public_exponent (int, optional): The public exponent to use. Defaults to 65537.
 
         Returns:
-            RSAPrivateKey: Instance of RSAPrivateKey
+            RsaPrivateKey: Instance of RsaPrivateKey
         """
         return cls.from_class(
             _RSA.generate_private_key(
@@ -579,7 +590,7 @@ class RSAPrivateKey(PrivateKey):
         return self.key.sign(data, _PADDING.PKCS1v15(), hash_alg.value[1]())
 
 
-class DSAPublicKey(PublicKey):
+class DsaPublicKey(PublicKey):
     """
     Class for holding DSA public keys
     """
@@ -602,7 +613,7 @@ class DSAPublicKey(PublicKey):
 
     @classmethod
     # pylint: disable=invalid-name
-    def from_numbers(cls, p: int, q: int, g: int, y: int) -> "DSAPublicKey":
+    def from_numbers(cls, p: int, q: int, g: int, y: int) -> "DsaPublicKey":
         """
         Create a DSA public key from public numbers and parameters
 
@@ -613,7 +624,7 @@ class DSAPublicKey(PublicKey):
             y (int): The public number Y
 
         Returns:
-            DSAPublicKey: An instance of DSAPublicKey
+            DsaPublicKey: An instance of DsaPublicKey
         """
         return cls(
             key=_DSA.DSAPublicNumbers(
@@ -640,7 +651,7 @@ class DSAPublicKey(PublicKey):
             ) from InvalidSignature
 
 
-class DSAPrivateKey(PrivateKey):
+class DsaPrivateKey(PrivateKey):
     """
     Class for holding DSA private keys
     """
@@ -648,15 +659,15 @@ class DSAPrivateKey(PrivateKey):
     def __init__(self, key: _DSA.DSAPrivateKey):
         super().__init__(
             key=key,
-            public_key=DSAPublicKey(key.public_key()),
+            public_key=DsaPublicKey(key.public_key()),
             private_numbers=key.private_numbers(),
         )
 
     @classmethod
     # pylint: disable=invalid-name,too-many-arguments
-    def from_numbers(cls, p: int, q: int, g: int, y: int, x: int) -> "DSAPrivateKey":
+    def from_numbers(cls, p: int, q: int, g: int, y: int, x: int) -> "DsaPrivateKey":
         """
-        Creates a new DSAPrivateKey object from parameters and public/private numbers
+        Creates a new DsaPrivateKey object from parameters and public/private numbers
 
         Args:
             p (int): P parameter, the prime modulus
@@ -678,13 +689,13 @@ class DSAPrivateKey(PrivateKey):
         )
 
     @classmethod
-    def generate(cls) -> "DSAPrivateKey":
+    def generate(cls) -> "DsaPrivateKey":
         """
         Generate a new DSA private key
         Key size is fixed since OpenSSH only supports 1024-bit DSA keys
 
         Returns:
-            DSAPrivateKey: An instance of DSAPrivateKey
+            DsaPrivateKey: An instance of DsaPrivateKey
         """
         return cls.from_class(_DSA.generate_private_key(key_size=1024))
 
@@ -701,7 +712,7 @@ class DSAPrivateKey(PrivateKey):
         return self.key.sign(data, _HASHES.SHA1())
 
 
-class ECDSAPublicKey(PublicKey):
+class EcdsaPublicKey(PublicKey):
     """
     Class for holding ECDSA public keys
     """
@@ -725,7 +736,7 @@ class ECDSAPublicKey(PublicKey):
     # pylint: disable=invalid-name
     def from_numbers(
         cls, curve: Union[str, _ECDSA.EllipticCurve], x: int, y: int
-    ) -> "ECDSAPublicKey":
+    ) -> "EcdsaPublicKey":
         """
         Create an ECDSA public key from public numbers and parameters
 
@@ -735,7 +746,7 @@ class ECDSAPublicKey(PublicKey):
             y (int): The affine Y component of the public point
 
         Returns:
-            ECDSAPublicKey: An instance of ECDSAPublicKey
+            EcdsaPublicKey: An instance of EcdsaPublicKey
         """
         if not isinstance(curve, _ECDSA.EllipticCurve) and curve not in ECDSA_HASHES:
             raise _EX.InvalidCurveException(
@@ -773,7 +784,7 @@ class ECDSAPublicKey(PublicKey):
             ) from InvalidSignature
 
 
-class ECDSAPrivateKey(PrivateKey):
+class EcdsaPrivateKey(PrivateKey):
     """
     Class for holding ECDSA private keys
     """
@@ -781,7 +792,7 @@ class ECDSAPrivateKey(PrivateKey):
     def __init__(self, key: _ECDSA.EllipticCurvePrivateKey):
         super().__init__(
             key=key,
-            public_key=ECDSAPublicKey(key.public_key()),
+            public_key=EcdsaPublicKey(key.public_key()),
             private_numbers=key.private_numbers(),
         )
 
@@ -791,7 +802,7 @@ class ECDSAPrivateKey(PrivateKey):
         cls, curve: Union[str, _ECDSA.EllipticCurve], x: int, y: int, private_value: int
     ):
         """
-        Creates a new ECDSAPrivateKey object from parameters and public/private numbers
+        Creates a new EcdsaPrivateKey object from parameters and public/private numbers
 
         Args:
             curve Union[str, _ECDSA.EllipticCurve]: Curve used by the key
@@ -830,7 +841,7 @@ class ECDSAPrivateKey(PrivateKey):
             curve (EcdsaCurves): Which curve to use. Default secp521r1
 
         Returns:
-            ECDSAPrivateKey: An instance of ECDSAPrivateKey
+            EcdsaPrivateKey: An instance of EcdsaPrivateKey
         """
         return cls.from_class(_ECDSA.generate_private_key(curve=curve.value))
 
@@ -848,7 +859,7 @@ class ECDSAPrivateKey(PrivateKey):
         return self.key.sign(data, _ECDSA.ECDSA(curve_hash))
 
 
-class ED25519PublicKey(PublicKey):
+class Ed25519PublicKey(PublicKey):
     """
     Class for holding ED25519 public keys
     """
@@ -865,7 +876,7 @@ class ED25519PublicKey(PublicKey):
         )
 
     @classmethod
-    def from_raw_bytes(cls, raw_bytes: bytes) -> "ED25519PublicKey":
+    def from_raw_bytes(cls, raw_bytes: bytes) -> "Ed25519PublicKey":
         """
         Load an ED25519 public key from raw bytes
 
@@ -873,7 +884,7 @@ class ED25519PublicKey(PublicKey):
             raw_bytes (bytes): The raw bytes of the key
 
         Returns:
-            ED25519PublicKey: Instance of ED25519PublicKey
+            Ed25519PublicKey: Instance of Ed25519PublicKey
         """
         if b"ssh-ed25519" in raw_bytes:
             id_length = unpack(">I", raw_bytes[:4])[0] + 8
@@ -902,16 +913,16 @@ class ED25519PublicKey(PublicKey):
             ) from InvalidSignature
 
 
-class ED25519PrivateKey(PrivateKey):
+class Ed25519PrivateKey(PrivateKey):
     """
     Class for holding ED25519 private keys
     """
 
     def __init__(self, key: _ED25519.Ed25519PrivateKey):
-        super().__init__(key=key, public_key=ED25519PublicKey(key.public_key()))
+        super().__init__(key=key, public_key=Ed25519PublicKey(key.public_key()))
 
     @classmethod
-    def from_raw_bytes(cls, raw_bytes: bytes) -> "ED25519PrivateKey":
+    def from_raw_bytes(cls, raw_bytes: bytes) -> "Ed25519PrivateKey":
         """
         Load an ED25519 private key from raw bytes
 
@@ -919,19 +930,19 @@ class ED25519PrivateKey(PrivateKey):
             raw_bytes (bytes): The raw bytes of the key
 
         Returns:
-            ED25519PrivateKey: Instance of ED25519PrivateKey
+            Ed25519PrivateKey: Instance of Ed25519PrivateKey
         """
         return cls.from_class(
             _ED25519.Ed25519PrivateKey.from_private_bytes(data=raw_bytes)
         )
 
     @classmethod
-    def generate(cls) -> "ED25519PrivateKey":
+    def generate(cls) -> "Ed25519PrivateKey":
         """
         Generates a new ED25519 Private Key
 
         Returns:
-            ED25519PrivateKey: Instance of ED25519PrivateKey
+            Ed25519PrivateKey: Instance of Ed25519PrivateKey
         """
         return cls.from_class(_ED25519.Ed25519PrivateKey.generate())
 
